@@ -33,7 +33,7 @@ from .phonemetadata import PhoneMetadata
 from .phonenumberutil import _VALID_PUNCTUATION, REGION_CODE_FOR_NON_GEO_ENTITY
 from .phonenumberutil import _PLUS_SIGN, _PLUS_CHARS_PATTERN
 from .phonenumberutil import _extract_country_code, region_code_for_country_code
-from .phonenumberutil import country_code_for_region
+from .phonenumberutil import country_code_for_region, normalize_diallable_chars_only
 from .phonenumberutil import _formatting_rule_has_first_group_only
 
 # Character used when appropriate to separate a prefix, such as a long NDD or
@@ -43,16 +43,6 @@ _EMPTY_METADATA = PhoneMetadata(id=unicod(""),
                                 international_prefix=unicod("NA"),
                                 register=False)
 
-# A pattern that is used to match character classes in regular expressions. An
-# example of a character class is [1-4].
-_CHARACTER_CLASS_PATTERN = re.compile(unicod("\\[([^\\[\\]])*\\]"))
-# Any digit in a regular expression that actually denotes a digit. For
-# example, in the regular expression 80[0-2]\d{6,10}, the first 2 digits (8
-# and 0) are standalone digits, but the rest are not.
-# Two look-aheads are needed because the number following \\d could be a
-# two-digit number, since the phone number can be as long as 15 digits.
-_STANDALONE_DIGIT_PATTERN = re.compile(unicod("\\d(?=[^,}][^,}])"))
-
 # A set of characters that, if found in a national prefix formatting rules, are an indicator to
 # us that we should separate the national prefix from the number when formatting.
 _NATIONAL_PREFIX_SEPARATORS_PATTERN = re.compile("[- ]")
@@ -62,9 +52,12 @@ _NATIONAL_PREFIX_SEPARATORS_PATTERN = re.compile("[- ]")
 # the format element under number_format contains groups of the dollar sign
 # followed by a single digit, separated by valid phone number
 # punctuation. This prevents invalid punctuation (such as the star sign in
-# Israeli star numbers) getting into the output of the AYTF.
+# Israeli star numbers) getting into the output of the AYTF. We require that
+# the first group is present in the output pattern to ensure no data is lost
+# while formatting; when we format as you type, this should always be the case.
 _ELIGIBLE_FORMAT_PATTERN = re.compile(unicod("[") + _VALID_PUNCTUATION + unicod("]*") +
-                                      unicod("(\\\\\\d") + unicod("[") + _VALID_PUNCTUATION + unicod("]*)+"))
+                                      unicod("\\\\1") + unicod("[") + _VALID_PUNCTUATION + unicod("]*") +
+                                      unicod("(\\\\\\d") + unicod("[") + _VALID_PUNCTUATION + unicod("]*)*"))
 
 # This is the minimum length of national number accrued that is required to
 # trigger the formatter. The first element of the leading_digits_pattern of each
@@ -185,17 +178,6 @@ class AsYouTypeFormatter(object):
 
     def _create_formatting_template(self, num_format):
         number_pattern = num_format.pattern
-
-        # The formatter doesn't format numbers when number_pattern contains
-        # "|", e.g.  (20|3)\d{4}. In those cases we quickly return.
-        if number_pattern.find('|') != -1:
-            return False
-
-        # Replace anything in the form of [..] with \d
-        number_pattern = re.sub(_CHARACTER_CLASS_PATTERN, unicod("\\\\d"), number_pattern)
-
-        # Replace any standalone digit (not the one in d{}) with \d
-        number_pattern = re.sub(_STANDALONE_DIGIT_PATTERN, unicod("\\\\d"), number_pattern)
         self.formatting_template = U_EMPTY_STRING
         temp_template = self._get_formatting_template(number_pattern, num_format.format)
         if len(temp_template) > 0:
@@ -415,7 +397,18 @@ class AsYouTypeFormatter(object):
                 else:
                     self._should_add_space_after_national_prefix = bool(_NATIONAL_PREFIX_SEPARATORS_PATTERN.search(number_format.national_prefix_formatting_rule))
                 formatted_number = re.sub(num_re, number_format.format, self._national_number)
-                return self._append_national_number(formatted_number)
+                # Check that we did not remove nor add any extra digits when we matched
+                # this formatting pattern. This usually happens after we entered the last
+                # digit during AYTF. Eg: In case of MX, we swallow mobile token (1) when
+                # formatted but AYTF should retain all the number entered and not change
+                # in order to match a format (of same leading digits and length) display
+                # in that way.
+                full_output = self._append_national_number(formatted_number)
+                formatted_number_digits_only = normalize_diallable_chars_only(full_output)
+                if formatted_number_digits_only == self._accrued_input_without_formatting:
+                    # If it's the same (i.e entered number and format is same), then it's
+                    # safe to return this in formatted number as nothing is lost / added.
+                    return full_output
         return U_EMPTY_STRING
 
     def get_remembered_position(self):
